<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DB;

class EntetesController extends Controller
{
    private $table_name = "t_entete";
    private $query;
    private $selections = array();
    private $group_by = array();
    private $is_form = false;

    private function getStruct(){
        $query = "SELECT CASE WHEN DATA_TYPE = 'varchar' THEN 'readonly' ELSE DATA_TYPE END as type, COLUMN_NAME as `key`, CASE WHEN COLUMN_KEY = 'PRI' THEN 0 ELSE CASE WHEN COLUMN_KEY = 'MUL' THEN 1 ELSE 2 END END as enabled, CASE WHEN IS_NULLABLE = 'NO' THEN 1 ELSE 0 END as required 
        FROM INFORMATION_SCHEMA.COLUMNS
        WHERE 
             TABLE_NAME = '".$this->table_name."' AND COLUMN_NAME <> 'created_date'";

        return DB::select(DB::raw($query));
    }

    public function structures(){
        return $this->getStruct();
    }

    private function getSize(){
        return DB::table($this->table_name)->count();
    }

    public function all(Request $request){
        $limit = $request->input('limit');
        $offset = $request->input('offset');
        $order = $request->input('order') ? $request->input('order') : 'DESC';
        $by = ($request->input('by') ? $request->input('by') : 'id');
        $filter_key = $request->input('filter_key');
        $value = $request->input('filter_val');
        $joins = json_decode($request->input('joins'));

        $selections = $this->getStruct();        

        $this->query = DB::table($this->table_name);

        $size = $this->query->count();
       
        if(isset($joins)){
            $this->join($joins, $this->table_name);
            $this->query->select($this->table_name.'.*', DB::raw(implode($this->selections,',')));            
        }else{
            $this->query->select($this->table_name.'.*');
        }

        if(str_contains($by, '_relation')){
            $by = $this->format_selection($by);
        }else
            $by = $this->table_name.'.'.$by;
        
        if(isset($filter_key) && isset($value)){            
            $keys = explode(',',$filter_key);
            $f = true;
            foreach($keys as $column){
                $column = $this->table_name.'.'.$column;
                if($f)
                    $this->query->where($column, 'like', '%'.$value.'%');
                else    
                    $this->query->orWhere($column, 'like', '%'.$value.'%');
                $f= false;
            }
            $size = $this->query->count();
        }else{
            $this->query->groupBy($this->table_name.'.id');
        }
        $this->query->offset($offset)->limit($limit);
        $this->query->orderBy($by, $order);

        $results = $this->query->get()->toArray();

        $results = array_map(function($item){
            if(array_key_exists('submenu', (array)$item)){                
                switch ($item->menu) {
                    case 'annuaire':
                        if(isset($item->submenu2) && $item->submenu2 != ''){
                            $item->submenu = DB::table('t_category as c1')->leftJoin('t_category as c2', 'c2.id', '=', 'c1.id_parent')->select(DB::raw("CONCAT(c2.nom,' > ', c1.nom) as nom"))->where('c1.id', $item->submenu)->first();
                            $item->submenu = (isset($item->submenu->nom)) ? $item->submenu->nom : "";
                        }
                        else{
                            $item->submenu = DB::table('t_category')->select(DB::raw("nom"))->where('id', $item->submenu)->first();
                            $item->submenu = (isset($item->submenu->nom)) ? $item->submenu->nom : "";
                        }
                            break;
                    case 'agenda':
                        $item->submenu = DB::table('t_agenda_categories')->select('name')->where('id', $item->submenu)->first();
                        $item->submenu = (isset($item->submenu->name)) ? $item->submenu->name : "";
                        break;
                    case 'cinema':
                        $item->submenu = DB::table('t_cine')->select('nom')->where('id', $item->submenu)->first();
                        $item->submenu = (isset($item->submenu->nom)) ? $item->submenu->nom : "";
                        break;    
                    default:
                        # code...
                        break;
                }
            };
            return $item;
        },$results);
       
        if($offset == 0){            
            return array('items' => $results, 'fields' => $selections, 'size' => $size);
        }else{
            return array('items' => $results);
        }

    }

    private function join($joins, $table_parent){
        $select = '_relation';
        foreach($joins as $j => $join){
            $select .= '_'.$join->table;
            $this->query->join($join->table, $join->table.'.'.$join->on , '=', $table_parent.'.'.$join->key);
            if(isset($join->join))
                $this->join($join->join,$join->table);
            if(isset($join->select)){
                $s = ($this->is_form ? 'id' : $join->select);
                $select .= '_'. $s;
                array_push( $this->selections ,'GROUP_CONCAT('.$join->table.'.'.$s.') as '.$select );
                $select = '_relation';
            }
        }
    }

    private function format_selection($val){
        $strs = explode('_',$val);
        $val = $strs[count($strs)-1];
        unset($strs[count($strs)-1]);
        unset($strs[0]);
        unset($strs[1]);

        $val = implode($strs,'_').'.'.$val;
        return $val;
    }

    public function selection(){
        $orderby = array_reduce($this->getStruct(), function($val, $item){
            if($val)
                return $val;
            else if($item->enabled == 2 && $item->required == 1 && $item->type == 'varchar') 
                $val = $item->key;
            return $val;
        });
        $items = DB::table($this->table_name)->orderBy($orderby, 'ASC')->get();

        return $items;
    }

    public function byId($id, Request $request){
        $this->is_form = true;
        $this->query = DB::table($this->table_name);

        $joins = json_decode($request->input('joins'));
        if(isset($joins)){
            $this->join($joins, $this->table_name);
            $this->query->select($this->table_name.'.*', DB::raw(implode($this->selections,',')));            
        }else{
            $this->query->select($this->table_name.'.*');
        }
        
        $this->query->where($this->table_name.'.id', $id);
        $item = (array)$this->query->first();

        $keys= array_keys($item);
        foreach($keys as $key){
            if(preg_match('/^_relation/',$key)){
                $item[$key] =  explode(',',$item[$key]);                     
            }
        }

        $item = (object)$item;
        switch ($item->menu) {
            case 'annuaire':
                if(isset($item->submenu2) && $item->submenu2 != ''){
                    //$item->submenu = DB::table('t_category as c1')->leftJoin('t_category as c2', 'c2.id', '=', 'c1.id_parent')->select(DB::raw("CONCAT(c2.nom,' > ', c1.nom) as nom"))->where('c1.id', $item->submenu)->first()->nom;
                    $tmp = DB::table('t_category')->select(DB::raw("nom"))->where('id', $item->submenu)->first()->nom;
                    $item->submenu = DB::table('t_category')->select(DB::raw("nom"))->where('id', $item->submenu2)->first()->nom;
                    $item->submenu2 = $tmp;
                }else
                    $item->submenu = DB::table('t_category')->select(DB::raw("nom"))->where('id', $item->submenu)->first()->nom;
                break;
            case 'agenda':
                $item->submenu = DB::table('t_agenda_categories')->select('name')->where('id', $item->submenu)->first()->name;
                break;
            case 'cinema':
                $item->submenu = DB::table('t_cine')->select('nom')->where('id', $item->submenu)->first()->nom;
                break;    
            default:
                # code...
                break;
        }

        return (array)$item;
    }

    public function add(Request $request){
        $data = json_decode($request->input('data'));

        if(isset($data->submenu2) && $data->submenu2 != ''){
            $m1 = $data->submenu;
            $m2 = $data->submenu2;
            $data->submenu = $m2;
            $data->submenu2 = $m2;
        }

        $data = (array)$data;

        return DB::table($this->table_name)->insert($data);
    }

    public function update(Request $request){
        $data = json_decode($request->input('data'));
        $extras = $request->input('extra');
        $id = $data->id;
        unset($data->id);
        unset($data->menu);
        unset($data->submenu);
        unset($data->submenu2);
        $data = (array)$data;
        $keys= array_keys($data);

        $update = array();
        foreach($keys as $key){
            if(preg_match('/^_relation/',$key)){
                $id_update = $data[$key];                
                unset($data[$key]);  
                if(!isset($extras[$key]))
                    continue;
                $options = json_decode($extras[$key]);
                DB::table($options[0])->where($options[1], $id)->delete();
                $update[$options[1]] = $id;
                if(is_array($id_update)){
                    foreach($id_update as $sid){
                        $update[$options[2]] = $sid;
                        DB::table($options[0])->insert($update);
                    }
                }else{
                    $update[$options[2]] = $id_update;
                    DB::table($options[0])->insert($update);
                }
            }
        }

        return  DB::table($this->table_name)->where('id', $id)->update($data);
    }

    public function remove(Request $request){
        $id = $request->input('id');
        return  DB::table($this->table_name)->where('id', $id)->delete();
    }

    public function title_desc_keyword(Request $request){
        $menu = $request->input('menu');
        $submenu = $request->input('submenu');
        $submenu2 = $request->input('submenu2');

        switch ($menu) {
            case 'contact':
                $menu = 'services';
                break;
            case 'index':
                $menu = 'accueil';
                break;  
            default:
                # code...
                break;
        }
        
        $query = DB::table($this->table_name)->select('title','description','keywords')->where('menu', $menu);
        if(isset($submenu))
            $query->where('submenu', $submenu);        
        if(isset($submenu2))
            $query->where('submenu2', $submenu2);
        
        return (array)$query->first();
    }   

    private function importentete(){
        
        //Annuaire
        /*$query = "SELECT * FROM t_category";
        $_results = DB::select( DB::raw($query));

        $results = array();
        foreach($_results as $r => $res){
            if(isset($res->id_parent))
                DB::statement("INSERT INTO t_entete (menu, submenu, submenu2, title, description, keywords)  (SELECT 'annuaire', '" . $res->id . "', '" . $res->id_parent . "' , t.entete, t.mdescription, t.mkeywords FROM (SELECT DISTINCT(rubrique), sousmenu, entete, mdescription, mkeywords FROM toulou_entete WHERE menu = 'annuaire' AND rubrique = '" . str_replace("'", "''", $res->nom) . "' AND sousmenu = (SELECT DISTINCT(nom) FROM t_category where id = ".$res->id_parent.")) t )");
            else
                DB::statement("INSERT INTO t_entete (menu, submenu, submenu2, title, description, keywords)  (SELECT 'annuaire', '" . $res->id . "', '" . $res->id_parent . "' , t.entete, t.mdescription, t.mkeywords FROM (SELECT DISTINCT(rubrique), sousmenu, entete, mdescription, mkeywords FROM toulou_entete WHERE menu = 'annuaire' AND rubrique IS NULL AND sousmenu = '" . str_replace("'", "''", $res->nom) . "') t )");

        }

        //Agenda
        $query = "SELECT * FROM t_agenda_categories";
        $_results = DB::select( DB::raw($query));

        $results = array();
        foreach($_results as $r => $res){
            DB::statement("INSERT INTO t_entete (menu, submenu, title, description, keywords)  (SELECT 'agenda', '" . $res->id . "', t.entete, t.mdescription, t.mkeywords FROM (SELECT DISTINCT(rubrique), sousmenu, entete, mdescription, mkeywords FROM toulou_entete WHERE menu = 'agenda' AND rubrique IS NULL AND sousmenu = '" . str_replace("'", "''", $res->libelle) . "') t )");

        }*/

        //cinema
        /*$query = "SELECT * FROM t_cine";
        $_results = DB::select( DB::raw($query));

        $results = array();
        foreach($_results as $r => $res){
            DB::statement("INSERT INTO t_entete (menu, submenu, title, description, keywords)  (SELECT 'cinema', '" . $res->id . "', t.entete, t.mdescription, t.mkeywords FROM (SELECT DISTINCT(sousmenu), entete, mdescription, mkeywords FROM toulou_entete WHERE menu = 'cinema' AND rubrique IS NULL AND sousmenu = '" . str_replace("'", "''", $res->nom) . "') t )");

        }*/

        //services
        //DB::statement("INSERT INTO t_entete (menu, submenu, title, description, keywords)  (SELECT 'services', t.sousmenu, t.entete, t.mdescription, t.mkeywords FROM (SELECT DISTINCT(sousmenu), entete, mdescription, mkeywords FROM toulou_entete WHERE menu = 'services' AND rubrique IS NULL ) t )");


        
        return $_results;
    }
}